#! @@PERL@@

# hosts2sql.pl					Author: Rich West
#						Rich.West@wesmo.com
#
# This is a small script which will integrate a typical NIS-type hosts file
# into the MID databases within MySQL.  The format of the file simply needs
# to be :
# IP_Address Hostname Alias Alias Alias ...
#
# The aliases are optional, but the script can handle them.  It takes the first
# name as the primary name and the rest are treated as aliases.
#
# You can grab this information from your DNS files as well, but you have to
# massage the content of the files to be in the format listed above.
################################################################################


################################################################################
#
# The modules we require.
#
################################################################################
use Mysql;

&main();

################################################################################
#
# Make sure we are being invoked correctly
#
################################################################################
sub check_usage()
{
   &usage if ($#ARGV > 2);
   if (($#ARGV < 0) || (($#ARGV == 0) && ($ARGV[0] eq "-v")) )
   {
      $FILE = "/etc/hosts";
   }
   else
   {
      if ($ARGV[0] eq "-f")
      {
         if (-e $ARGV[1])
         {
            $FILE = $ARGV[1];
         }
         else
         {
            &usage();
         }
      }
      if ($ARGV[1] eq "-f")
      {
         if (-e $ARGV[2])
         {
            $FILE = $ARGV[2];
         }
         else
         {
            &usage();
         }
      }
   }
   
   $DEBUG = 1 if ( ($ARGV[0] eq "-v") || $ARGV[2] eq "-v");
}

################################################################################
#
# Our Usage Statement
#
################################################################################
sub usage()
{
   print STDERR "Usage: $0 [-v] [-f <filename>]\n\n";
   exit 0;
}

################################################################################
#
# Our Database Information
#
################################################################################
sub db_settings
{
   $host = "@@DBHOST@@";
   $dbname = "@@DB@@";
   $user = "@@DBUSER@@";
   $pass = "@@DBPASS@@";
}

################################################################################
#
# Connect to the Database
#
################################################################################
sub connectdb
{
   local($dbname) = @_;

   if (!($dbh = Mysql->Connect($host, $dbname, $user, $pass,)))
   {
      die "[connectdb] Database connect failed: $Mysql::db_errstr\n";
   }
}

sub insert
{
   local($ip,$host) = @_;
   local($qs, $sth);
   
   $qs="INSERT INTO name VALUES ('', '$ip', '$host', '')";
   $sth = $dbh->Query($qs)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$qs\n";
   $qs="SELECT id FROM name WHERE Machine_Name='$host'";
   $sth = $dbh->Query($qs)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$qs\n";
   @row=$sth->FetchRow;
   $qs = "INSERT INTO changes VALUES ('$row[0]', 'root', '00000000')\n";
   $sth = $dbh->Query($qs)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$qs\n";
   $qs = "INSERT INTO specs VALUES ('$row[0]', '', '', '', '', '')\n";
   $sth = $dbh->Query($qs)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$qs\n";
   $qs = "INSERT INTO owner VALUES ('$row[0]', '',  '', '')\n";
   $sth = $dbh->Query($qs)
      or warn "[insert] Query had some problem: $Mysql::db_errstr\n$qs\n";
   return($row[0]);
}

sub add_aliases
{
   local($id) = @_;
   local($qs);

   for ($i=0; $i<=$#aliases; $i++)
   {
      $_ = $aliases[$i];
      last if (/#/);
      $qs="INSERT INTO aliases VALUES ('$id', '$aliases[$i]')";
      $sth = $dbh->Query($qs)
         or warn "[add_aliases] Query had some problem: $Mysql::db_errstr\n$qs\n";
   }
   if (!$#aliases)
   {
      $qs="INSERT INTO aliases VALUES ('$id', '')";
      $sth = $dbh->Query($qs)
         or warn "[add_aliases] Query had some problem: $Mysql::db_errstr\n$qs\n";
   }
}

sub main
{
   &db_settings();
   &connectdb("@@DB@@");
   open (MASTER, "/etc/hosts");
   while (<MASTER>)
   {
      chop;
      next if ((/^#/));
      ($ip, $host, @aliases) = split(/\s+/, $_);
      print "Processing $host with $ip and $#aliases aliases...\n" if($DEBUG);
      $id=&insert($ip, $host);
      print "Got id $id...\n" if($DEBUG);
      &add_aliases($id);
   }
   close(MASTER);
}
